/**
* Filename: EventSourceBroker.java (in org.openbandy.event)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/

package org.openbandy.event;

import java.util.Enumeration;

import org.openbandy.util.DoubleHashtable;

/**
 * TODO describe purpose and usage
 * 
 * holds references to the registered EventSources and creates new ones
 * if necessary
 * 
 * hashing is done via adding up an integer
 * 
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 * 
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.0
 */
public class EventSourceBroker {

    /* a pool of references to the event sources, i.e.
     * index -> eventSource and vice versa */
    private DoubleHashtable eventSourcesPool;
    
    /* number of last entry */
    private int lastIndex = 0;
    
    /* Constructor */
    public EventSourceBroker(){
        eventSourcesPool = new DoubleHashtable();
    }
    
    /* *************** Getter and Setter ***************** */
    
    public Integer addEventSource(EventSource eventSource){
        if(! eventSourcesPool.containsKey(eventSource)){
            lastIndex++;
            eventSourcesPool.put(new Integer(lastIndex), eventSource);
            return new Integer(lastIndex);
        }else{
            return new Integer(0);
        }
    }
    
    public EventSource getEventSource(Integer index){
        return (EventSource)eventSourcesPool.getValueByKey(index);
    }
    
    public Integer getEventSourceIndex(EventSource eventSource){
        return (Integer)eventSourcesPool.getKeyByValue(eventSource);
    }
    
    public Enumeration getSources(){
        return eventSourcesPool.getValues();
    }
    
    public boolean removeEventSource(EventSource eventSource){
        if(eventSourcesPool.removeByValue(eventSource) != null){
            lastIndex--;
            return true;
        }else{
            return false;
        }
    }
    

    
    public EventSource getNewEventSourceInstance(Integer index){
        /* create new instance */
        EventSource oldEventSource = getEventSource(index);
        EventSource newEventSource = oldEventSource.newlyInstantiate();
        
        /* update sources pool */
        eventSourcesPool.removeByKey(index);
        eventSourcesPool.put(index, newEventSource);
        
        return newEventSource;
        
    }
}
