/**
 * Filename: BooleanRmsTest.java (in org.openbandy.example.test)
 * This file is part of the BandySerializer project.
 *
 * www.BandySerializer.org
 */

package org.openbandy.example.test;

import java.util.Vector;

import org.openbandy.io.Serializable;
import org.openbandy.io.SerializationBuffer;
import org.openbandy.io.SerializationIdentifiers;
import org.openbandy.io.Serializer;
import org.openbandy.io.rms.RmsStorage;
import org.openbandy.service.LogService;
import org.openbandy.service.SerializerService;
import org.openbandy.test.Test;

/**
 * TODO describe purpose and usage
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public class BooleanRmsTest extends Test {

	public BooleanRmsTest() {
		super("Boolean Rms Serialization");
	}

	public void run() {
		testStarted();

		/* create a test object containing several boolean values */
		BooleanTestObject booleanTestObject = new BooleanTestObject();
		booleanTestObject.setCorrectValues();

		/* store the bo in the rms */
		SerializerService.storeToRms(booleanTestObject);

		/* create a new object of this type and overload it with the values from the rms */
		BooleanTestObject clonedBooleanTestObject = new BooleanTestObject();
		SerializerService.loadFromRms(clonedBooleanTestObject, booleanTestObject.getIdValue());

		/* delete the object in the rms */
		if(!RmsStorage.getInstance().delete(booleanTestObject)){
			LogService.warn(this, "Could not delete object from rms");
			testsScreen.setTestFailed(this);
		}
		else{
			/* check if the object is actually deleted from the rms */
			if (RmsStorage.getInstance().existsInStorage(booleanTestObject) != 0) {
				LogService.warn(this, "Object not deleted from rms");
				testsScreen.setTestFailed(this);
			}
			else {
				/* compare original object with clone */
				if(!clonedBooleanTestObject.isEqual(booleanTestObject)) {
					LogService.warn(this, "Clone is not equal");
					testsScreen.setTestFailed(this);
				}
				else {
					testsScreen.setTestPassed(this);
				}
			}
		}
		testFinished();
	}
}

class BooleanTestObject implements Serializable {

	private int booleanTestObjectId = SerializationIdentifiers.NO_ID;

	/* Test Values */
	boolean falseBoolean;
	boolean trueBoolean;
	boolean falseBooleanIntRef;
	boolean trueBooleanIntRef;

	public BooleanTestObject(){
		falseBoolean = true;
		trueBoolean = false;
		falseBooleanIntRef = true;
		trueBooleanIntRef = false;
	}

	public void setCorrectValues(){
		falseBoolean = false;
		trueBoolean = true;
		falseBooleanIntRef = false;
		trueBooleanIntRef = true;
	}

	public void serialize(Serializer serializer, SerializationBuffer buffer) {
		buffer.container().addBoolean("falseBoolean", falseBoolean);
		buffer.container().addBoolean("trueBoolean", trueBoolean);

		buffer.container().addInt("falseBooleanIntRef", (falseBooleanIntRef ? 1 : 0));
		buffer.container().addInt("trueBooleanIntRef", (trueBooleanIntRef ? 1 : 0));
	}

	public void deserialize(Serializer serializer, SerializationBuffer buffer) {
		falseBoolean = buffer.container().getBoolean("falseBoolean");
		trueBoolean = buffer.container().getBoolean("trueBoolean");

		falseBooleanIntRef = (buffer.container().getInt("falseBooleanIntRef") == 1);
		trueBooleanIntRef = (buffer.container().getInt("trueBooleanIntRef") == 1);
	}

	public String getIdName() {
		return "booleanTestObjectId";
	}

	public int getIdValue() {
		return booleanTestObjectId;
	}

	public void setIdValue(int idValue) {
		booleanTestObjectId = idValue;
	}

	public void getAttributeNames(Vector attributes) {
		attributes.addElement("falseBoolean");
		attributes.addElement("trueBoolean");
		attributes.addElement("falseBooleanIntRef");
		attributes.addElement("trueBooleanIntRef");
	}

	public void getReferences(Vector references) {
	}

	public void getReferencedIdNames(Vector referenceNames) {
	}

	public String getLegibleString() {
		return new String(falseBoolean+":"+trueBoolean+":"+falseBooleanIntRef+":"+trueBooleanIntRef);
	}

	public boolean isEqual(Serializable serializableObject) throws ClassCastException {
		return isEqualShallow(serializableObject);
	}

	public boolean isEqualShallow(Serializable serializableObject) throws ClassCastException {
		BooleanTestObject comparObject = (BooleanTestObject)serializableObject;
		return ( (comparObject.falseBoolean == this.falseBoolean)
				&& (comparObject.falseBooleanIntRef == this.falseBooleanIntRef)
				&& (comparObject.trueBoolean == this.trueBoolean)
				&& (comparObject.trueBooleanIntRef == this.trueBooleanIntRef) );
	}

	public Serializable getNewInstance(){
		return new BooleanTestObject();
	}

	public Serializable getCopy() {
		BooleanTestObject clone = new BooleanTestObject();
		clone.falseBoolean = this.falseBoolean;
		clone.falseBooleanIntRef = this.falseBooleanIntRef;
		clone.trueBoolean = this.trueBoolean;
		clone.trueBooleanIntRef = this.trueBooleanIntRef;
		return clone;
	}
}

