/**
 * Filename: EventSourceSimulation.java (in org.openbandy.example.test)
 * This file is part of the BandyEvents project.
 *
 * www.BandyEvents.org
 */

package org.openbandy.example.test;

import java.util.Enumeration;
import java.util.Vector;

import org.openbandy.event.EventSource;
import org.openbandy.event.EventSourceOwner;
import org.openbandy.service.EventService;
import org.openbandy.service.LogService;

/**
 * TODO describe purpose and usage
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public class EventSourceSimulation extends EventSource {

	/* monitor object to inform others when I'm finished sending events */
	private AllEventsSentMonitor monitor;

	/* A vector of events that will be fired during the simulation */
	private Vector simulatedEvents = new Vector();

	/* interval between events in milliseconds */
	private final static int INTERVAL = 500;

	/* number of events that will be sent during simulation */
	private int numberOfEventsToSimulate = 0;

	public EventSourceSimulation(AllEventsSentMonitor monitor, EventSourceOwner owner, int numberOfEventsToSimulate) {
		super("Event Source Simulation", owner);
		this.monitor = monitor;

		/* register as sink at the event router */
		EventService.registerSource(this);

		/* create events that will be sent during simulation */
		for(int i=0; i<numberOfEventsToSimulate; i++){
			simulatedEvents.addElement(new TestEvent("I'm event number "+(i+1)));
		}
	}

	public Vector getProvidedEventTypes() {
		Vector providedEventTypes = new Vector();
		providedEventTypes.addElement("TestEvent");
		return providedEventTypes;
	}

	public EventSource newlyInstantiate() {
		EventSourceSimulation newInstance = new EventSourceSimulation(this.monitor, this.owner, this.numberOfEventsToSimulate);
		owner.setNewInstance(newInstance);
		return newInstance;
	}


	public void run() {

		/* ***** simulate event source ***** */
		Enumeration elements = simulatedEvents.elements();
		while(elements.hasMoreElements()){
			TestEvent event = (TestEvent)elements.nextElement();
			LogService.debug(this, "Sending Event: "+event.getEventMessage());
			EventService.delegate(event);

			/* wait before sending next event */
			try{
				sleep(INTERVAL);
			}catch(InterruptedException ie){
				ie.printStackTrace();
			}
		}

		/* inform tests that all events are sent */
		monitor.setAllSent();

	}

	public void stopEventSource(String eventShortClassName) {
		consumedEvents.removeElement(eventShortClassName);
		if(consumedEvents.isEmpty()){
			this.running = false;
			LogService.debug(this, "Stoped EventSource : "+getName());
		}
	}

	public void stopEventSource() {
		consumedEvents.removeAllElements();
		this.running = false;
		LogService.debug(this, "Stoped EventSource : "+getName());

	}

}
