/**
* Filename: PingPongTest.java (in org.openbandy.example.test)
* This file is part of the OpenBandy project.
*
* www.openbandy.org
*/

package org.openbandy.example.test;

import org.openbandy.example.pingpong.PingPongMessage;
import org.openbandy.example.pingpong.PingPongProtocol;
import org.openbandy.io.Message;
import org.openbandy.pref.Preferences;
import org.openbandy.pref.UnknownPreferenceException;
import org.openbandy.service.LogService;
import org.openbandy.service.MessageService;
import org.openbandy.test.Test;

/**
 * TODO describe purpose and usage
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public class PingPongTest extends Test {

	/* Constructor */
	public PingPongTest() {
		super("Ping Pong (Client-Server)");
	}
	
	public void run() {
		testStarted();
		
		String username = "";
		
		/* read user name from preferences */
		try {
			username = Preferences.getValue("Username");
		} catch (UnknownPreferenceException upe) {
			LogService.error(this, "Preference 'Username' not known", upe);
			username = "unknown";
		}
		
		try {
			/* read server host address from preferences */
			String host = Preferences.getValue("Server Address");
			
			/* create a ping pong request */
			PingPongProtocol protocol = new PingPongProtocol();
			Message request = protocol.createMessage(username, "Ping", 1);
			
			/* send it to the server */
			Message respsone = MessageService.sendMessage(host, protocol, request);
			
			/* the test failed if we do not get a response from the server */
			if(respsone == null){
				testsScreen.setTestFailed(this);
			}
			else {
				try{
					/* read the ping pong message from the response */
					PingPongMessage pingPongResponse = (PingPongMessage)respsone.getContent();
					
					/* check if serve counter is increased */
					if(pingPongResponse.getServeCounter() == 2){
						LogService.info(this, "Got "+pingPongResponse.getMessageString()+" from server");
						testsScreen.setTestPassed(this);
					}
					else{
						LogService.info(this, "Got "+pingPongResponse.getMessageString()+" from server, but serve counter was not increased");
						testsScreen.setTestFailed(this);
					}
				}
				catch(ClassCastException cce){
					LogService.error(this, "Response is not a ping pong message", cce);
					testsScreen.setTestFailed(this);
				}
			}			
		} catch (UnknownPreferenceException upe) {
			LogService.error(this, "Preference 'Server Address' not known", upe);
			testsScreen.setTestFailed(this);
		}
		
		testFinished();
	}
}
