/**
* Filename: StreamSerializerTest.java (in org.openbandy.example.test)
* This file is part of the BandySerializer project.
*
* www.BandySerializer.org
*/

package org.openbandy.example.test;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import org.openbandy.example.Car;
import org.openbandy.example.EuropeanSportscar;
import org.openbandy.io.stream.StreamSerializer;
import org.openbandy.service.LogService;
import org.openbandy.service.SerializerService;
import org.openbandy.test.Test;

/**
 * TODO describe purpose and usage
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public class StreamSerializerTest extends Test {

	public StreamSerializerTest() {
		super("Stream Serializer");
	}

	public void run() {
		try{
			
			testStarted();
			
			EuropeanSportscar car = TestDataGenerator.createDummyEuropeanSportscar();			
						
			/* ***** Testing serialization with no compression ***** */
			
			/* serialize into a stream using no compression */
			ByteArrayOutputStream baos = SerializerService.getXmlStream(car, StreamSerializer.COMPRESSION_NONE);
			LogService.info(this, "Size of steam using 'NO_COMPRESSION': "+baos.size());
			
			//TODO remove debug code
			System.out.println("No compression: size="+baos.size()+" \n"+baos+"\n\n");
			
			/* deserialize the object from the stream */
			ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
			Car clonedCar = (Car)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized object with original */
			if(!clonedCar.isEqual(car)) {
				LogService.warn(this, "Stream serializer without compression failed");
				testsScreen.setTestFailed(this);
				return;
			}
			else{
				testsScreen.setTestPassed(this);
			}
			
			/* ***** Testing tokenization ***** */
			
			/* serialize into a stream using token compression */
			baos = SerializerService.getXmlStream(car, StreamSerializer.COMPRESSED_TOKEN);
			LogService.info(this, "Size of steam using 'COMPRESSED_TOKEN': "+baos.size());
			
			//TODO remove debug code
			System.out.println("Tokenized: size="+baos.size()+" \n"+baos+"\n\n");
			
			/* deserialize the object from the stream */
			bais = new ByteArrayInputStream(baos.toByteArray());
			clonedCar = (Car)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized object with original */
			if(!clonedCar.isEqual(car)) {
				LogService.warn(this, "Stream serializer using using tokens failed");
				testsScreen.setTestFailed(this);
				return;
			}
			
			/* ***** Testing zip compression ***** */
			
			/* serialize into a stream using zip compression */
			baos = SerializerService.getXmlStream(car, StreamSerializer.COMPRESSED_ZIP);
			LogService.info(this, "Size of steam using 'COMPRESSED_ZIP': "+baos.size());
			
			//TODO remove debug code
			System.out.println("Zipped: size="+baos.size()+" \n"+baos+"\n\n");
			
			/* deserialize the object from the stream */
			bais = new ByteArrayInputStream(baos.toByteArray());
			clonedCar = (Car)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized object with original */
			if(!clonedCar.isEqual(car)) {
				LogService.warn(this, "Stream serializer using zip compression failed");
				testsScreen.setTestFailed(this);
				return;
			}
			
			/* ***** Testing highest compression rate (tokenize and zip) ***** */
			
			/* serialize into a stream using the highest compression rate */
			baos = SerializerService.getXmlStream(car, StreamSerializer.COMPRESSED_BEST);
			LogService.info(this, "Size of steam using 'COMPRESSED_BEST': "+baos.size());
			
			//TODO remove debug code
			System.out.println("Tokenized & Zipped: size="+baos.size()+" \n"+baos+"\n\n");
			
			/* deserialize the object from the stream */
			bais = new ByteArrayInputStream(baos.toByteArray());
			clonedCar = (Car)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized object with original */
			if(!clonedCar.isEqual(car)) {
				LogService.warn(this, "Stream serializer test with best compression failed");
				testsScreen.setTestFailed(this);
				return;
			}
		}
		catch(Exception e){
			testsScreen.setTestFailed(this);
			LogService.error(this, "Stream serializer test failed", e);
		}
		
		testFinished();
		
	}

}
