/**
 * Filename: StorageValueContainer.java (in org.openbandy.io.storage)
 * This is part of the OpenBandy project.
 * 
 * www.openbandy.org
 */

package org.openbandy.io.storage;

import org.openbandy.io.Serializable;
import org.openbandy.io.SerializableVector;
import org.openbandy.io.ValueContainer;
import org.openbandy.service.LogService;


/**
 * An implementation of the ValueContainer interface for storage
 * purposes. It uses ObjectEntry objects to set and get attribute values.
 *
 * <br><br>
 * (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.0
 */
public abstract class StorageValueContainer extends ValueContainer {

	/* internal data structure for store purposes */
	public ObjectEntry objectEntry;

	/* string representation of object ids for use with vector */
	private ObjectIdString objectIdString = null;


	public void setObjectIdString(ObjectIdString objectIdString){
		this.objectIdString = objectIdString;
	}

	/* **** add **** */
	public void addString(String name, String value){
		if (assertObjectEntry()){
			objectEntry.add(name, value);
		}
	}

	public void addBoolean(String name, boolean value) {
		if(value){
			addInt(name, 1);
		}
		else{
			addInt(name, 0);
		}
	}

	/* **** get **** */

	public boolean getBoolean(String name) {
		int value = getInt(name);
		if(value == 1){
			return true;
		}
		else{
			return false;
		}
	}

	public int getIdForObject(Serializable serializable){
		if(objectIdString != null){
			return objectIdString.getNext();
		}else{

			/* determine foreign key name */
			String idName = "";
			if(serializable instanceof SerializableVector){
				idName = ((SerializableVector)serializable).getSerializationName();
			}else{
				idName = serializable.getIdName();
			}
			return getInt(idName);
		}
	}


	/* helper */
	protected boolean assertObjectEntry(){
		if (objectEntry == null){
			LogService.warn(this, "No objectEntry available.");
			return false;
		}
		return true;
	}

}
