/**
* Filename: StreamSerializationBuffer.java (in org.openbandy.io.stream)
* This file is part of the OpenBandy project.
* 
* www.openbandy.org
* 
*/

package org.openbandy.io.stream;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.util.Enumeration;
import java.util.Vector;

import org.kxml.io.AbstractXmlWriter;
import org.kxml.io.XmlWriter;
import org.kxml.parser.AbstractXmlParser;
import org.kxml.parser.ParseEvent;
import org.kxml.parser.XmlParser;
import org.kxml.wap.WbxmlParser;
import org.kxml.wap.WbxmlWriter;
import org.openbandy.io.Serializable;
import org.openbandy.io.SerializableVector;
import org.openbandy.io.SerializationBuffer;
import org.openbandy.service.LogService;



/**
* TODO describe purpose and usage
*
* <br><br>
* (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
*
* @author Philipp Bolliger (philipp@bolliger.name)
* @version 0.8
*/
public class StreamSerializationBuffer extends SerializationBuffer {
	
	/* Token tables */
	protected static Vector typeTable = new Vector();
	protected static Vector attributeTable = new Vector();
	protected static Vector attributeValueTable = new Vector();
	
	/* In- and Out-put streams */
	protected ByteArrayOutputStream outputStream;
	protected ByteArrayInputStream inputStream;
	
	/* Writer and Parser for kXML processing */
	protected AbstractXmlWriter writer;
	protected AbstractXmlParser parser;
	
	/* Keep last read event, see deserialize vector */
	protected ParseEvent lastReadEvent;
	
	/* Type of the actual stream */
	protected int streamType;
	
	/* Constructor */
	public StreamSerializationBuffer(){
		super();
		streamType = -1;
		writer = null;
		parser = null;
		outputStream = null;
		inputStream = null;
		lastReadEvent = null;
	}
	
	/* ******* Methods for StreamSerializer ******* */
	
	/**
	 * TODO comment!
	 *
	 * @return
	 */
	protected boolean isSetupUpForSerialzation(){
		return ((writer != null) && (currentValueContainer != null));
	}
	
	/**
	 * TODO comment!
	 *
	 * @return
	 */
	protected boolean isSetupForDeserialization(){
		return (parser != null);
	}
	
	/**
	 * TODO comment!
	 *
	 * @param serializable
	 * @param compression
	 */
	protected void setupForSerialization(Serializable serializable, int compression){
		
		/* *********** set output stream ************* */
		outputStream = new ByteArrayOutputStream();
		
		
		/* *********** set writer ************* */
		
		/* NO_COMPRESSION */
		if((compression | StreamSerializer.COMPRESSION_NONE) == 0){
			/* setup writer */
			writer = new XmlWriter(new OutputStreamWriter( outputStream ));
		}
		
		/* COMPRESSED_TOKEN */
		else if((compression & StreamSerializer.COMPRESSED_TOKEN) == StreamSerializer.COMPRESSED_TOKEN){
			/* get the names of every attribute in order to create tokens */
			attributeTable.removeAllElements();
			updateAttributeTableForObject(serializable);
			
			/* setup writer */
			try{
				writer = new WbxmlWriter( outputStream );                
				((WbxmlWriter)writer).setTagTable( 0, StreamSerializerHelper.getStringArray(typeTable));
				((WbxmlWriter)writer).setAttrStartTable( 0, StreamSerializerHelper.getStringArray(attributeTable));
				((WbxmlWriter)writer).setAttrValueTable( 0, StreamSerializerHelper.getStringArray(attributeValueTable));
			}catch(IOException ioe){
				LogService.error(this, "Can create writer", ioe);
			}  
		}
		
		/* at least create a simple writer */
		if(writer == null){
			writer = new XmlWriter(new OutputStreamWriter( outputStream ));
		}
		
		/* setup container */
		addContainer(new StreamValueContainer(writer));
	}

	/**
	 * TODO comment!
	 *
	 * @param inputStream
	 */
	protected void setupForDeserialization(ByteArrayInputStream inputStream){
		this.inputStream = inputStream;
		determineStreamType();
	}
	
	/**
	 * TODO comment!
	 *
	 */
	protected void setupXmlParser(){
		try {
			Reader reader = new InputStreamReader(inputStream);
			this.parser = new XmlParser(reader);
		} catch (IOException ioe) {
			LogService.error(this, "Could not create parser", ioe);
		}
	}
	
	/**
	 * TODO comment!
	 *
	 * @param serializable
	 */
	protected void setupWbxmlParser(Serializable serializable){
		/* get the names of every attribute in order to create tokens */
		attributeTable.removeAllElements();
		updateAttributeTableForObject(serializable);
		
		/* setup parser */
		try{
			parser = new WbxmlParser(inputStream);
			((WbxmlParser)parser).setTagTable( 0, StreamSerializerHelper.getStringArray(typeTable));
			((WbxmlParser)parser).setAttrStartTable( 0, StreamSerializerHelper.getStringArray(attributeTable));
			((WbxmlParser)parser).setAttrValueTable( 0, StreamSerializerHelper.getStringArray(attributeValueTable));
		}catch(IOException ioe){
			LogService.error(this, "Could not create parser", ioe);
		}
	}
	
	protected void closeWriter(){
		try {
			writer.close();
		}catch(IOException ioe){
			LogService.error(this, "Can not close writer", ioe);
		}
	}
	
	protected String getContentClassName(){
		if(streamType == StreamSerializer.COMPRESSION_NONE){
			return StreamSerializerHelper.readContentClassNameFromUncompressedStream(inputStream);
			
		}else if(streamType == StreamSerializer.COMPRESSED_TOKEN){
			return StreamSerializerHelper.readContentClassNameFromTokenizedStream(inputStream);
			
		}else if(streamType == StreamSerializer.COMPRESSED_ZIP){
			
		}else{
			
		}
		return "";
	}
	
	
	/* ************** Helper Methods ************** */
	
	private void determineStreamType(){
		int c = inputStream.read();
		if(c == 13){
			streamType = StreamSerializer.COMPRESSION_NONE;
		} else if(c == 1){
			streamType = StreamSerializer.COMPRESSED_TOKEN;
		}else if(c == 120){
			streamType = StreamSerializer.COMPRESSED_ZIP;
		}
		inputStream.reset();
	}
	
	private void updateAttributeTableForObject(Serializable serializable){
		/* get attribute names of serializable */
		serializable.getAttributeNames(attributeTable);
		
		/* add id name as attribute */
		attributeTable.addElement(serializable.getIdName());
		
		//TODO correct this hack (compress vector too)
		if( !(serializable instanceof SerializableVector) ){
			
			Vector references = new Vector();
			serializable.getReferences(references);
			
			Enumeration elements = references.elements();
			Serializable lastReference  = null;
			while(elements.hasMoreElements()){
				Serializable reference = (Serializable) elements.nextElement();
				if(reference != null){
					if(lastReference == null){
						lastReference = reference;
						updateAttributeTableForObject(reference);
					}
					/* only add attributes once per type */
					if( (lastReference!=null) && 
							(! reference.getClass().getName().equals(lastReference.getClass().getName())) ){
						updateAttributeTableForObject(reference);
					}
				}
			}
		}
	}
	

}
