/**
 * Filename: StreamSerializerHelper.java (in org.openbandy.io.stream)
 * This is part of the OpenBandy project.
 * 
 * www.openbandy.org
 */

package org.openbandy.io.stream;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Vector;

import org.openbandy.service.LogService;


/**
 * TODO describe purpose and usage
 *
 * <br><br>
 * (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.0
 */
public class StreamSerializerHelper {
	
	// TODO vereinfachen und verallgemeinern der methoden um den klassennamen zu erhalten -  jetzt viel code duplication
	
	//TODO make this flag a char seq that can not occur else (or make sure it can not be contained in the stream anyways)
	/* flag to denote the start of a class name */
	protected final static String CLASSNAME_FLAG = "SCN0";

	/**
	 * TODO comment!
	 *
	 * @param inputStream
	 * @return
	 */
	public static String readToken(InputStream inputStream){
		int ch;
		ByteArrayOutputStream buf = new ByteArrayOutputStream();
		try{
			while ((ch = inputStream.read()) >= 0) {
				if ((ch == 0)) {// only read to first token 0
					break;
				}
				buf.write(ch);
			}
		}catch(IOException ioe){
			LogService.error(StreamSerializerHelper.class, ioe.getMessage(), ioe);
		}
		return new String(buf.toByteArray());
	}

	/**
	 * TODO comment!
	 *
	 * @param inputStream
	 * @return
	 */
	public static String readContentClassNameFromUncompressedStream(InputStream inputStream){
		String contentClassName =  StreamSerializerHelper.readToken(inputStream).substring(5);

		/* trim class name */
		contentClassName = getTrimmedClassName(contentClassName);

		try{
			inputStream.reset();
		}catch(IOException ioe){
			LogService.error("StreamSerializerHelper", ioe.getMessage(), ioe);
		}
		return contentClassName;
	}

	/**
	 * TODO comment!
	 *
	 * @param inputStream
	 * @return
	 */
	public static String readContentClassNameFromTokenizedStream(InputStream inputStream) {
		/* the contentClassName is the second token */
		StreamSerializerHelper.readToken(inputStream);
		String contentClassName = StreamSerializerHelper.readToken(inputStream);

		/* trim class name */
		contentClassName = getTrimmedClassName(contentClassName);

		try{
			inputStream.reset();
		}catch(IOException ioe){
			LogService.error("StreamSerializerHelper", ioe.getMessage(), ioe);
		}
		return contentClassName;

	}

	/**
	 * TODO comment!
	 *
	 * @param inputStream
	 * @return
	 */
	public static String readClassNameFromUncompressedStream(InputStream inputStream) {
		String className = StreamSerializerHelper.readToken(inputStream);

		/* trim class name */
		className = getTrimmedClassName(className);

		try{
			inputStream.reset();
		}catch(IOException ioe){
			LogService.error("StreamSerializerHelper", ioe.getMessage(), ioe);
		}
		return className;
	}

	/**
	 * TODO comment!
	 * 
	 * TODO harden this better by skipping all the chars that are tokens to WbxmlXXXX
	 * 
	 * @param inputStream Tokenized XML in InputStream.
	 * @return The class name of the first object in the compressed xml stream.
	 * @throws IOException
	 */
	public static String readClassNameFromTokenizedStream(InputStream inputStream) {
		String className = StreamSerializerHelper.readToken(inputStream);
		
		/* trim class name */
		className = getTrimmedClassName(className);

		try{
			inputStream.reset();
		}catch(IOException ioe){
			LogService.error("StreamSerializerHelper", ioe.getMessage(), ioe);
		}
		return className;
	}

	/**
	 * TODO comment!
	 *
	 * @param v
	 * @return
	 */
	public static String[] getStringArray(Vector v){
		String[] result = new String[v.size()];
		Enumeration elements = v.elements();
		int i = 0;
		while(elements.hasMoreElements()){
			String value = (String) elements.nextElement();
			result[i++] = value;
		}
		return result;
	}
	
	/**
	 * TODO comment!
	 *
	 * @param untrimmedClassName
	 * @return
	 */
	private static String getTrimmedClassName(String untrimmedClassName){
		
		/* if the untrimmed class name does not contain the classname flag, return the empty string */
		if(untrimmedClassName.indexOf(CLASSNAME_FLAG) == -1){
			return "";
		}
		
		/* trim the class name, i.e. remove control chars */
		String trimmedClassName = untrimmedClassName.trim();
		
		/* remove the class name flag from the beginning */
		int classNameBegin = trimmedClassName.indexOf(CLASSNAME_FLAG)+CLASSNAME_FLAG.length();
		trimmedClassName = trimmedClassName.substring(classNameBegin);
		
		/* cut of the rest if there is any */
		int classNameEnd = trimmedClassName.indexOf(" ");
		if(classNameEnd > 0){
			return trimmedClassName.substring(0, classNameEnd);
		}
		else {
			return trimmedClassName;
		}
	}

}
