/**
* Filename: IncomingMessageEvent.java (in org.openbandy.net.client)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/


package org.openbandy.net.client;

import java.util.Vector;

import org.openbandy.event.Event;
import org.openbandy.io.Serializable;
import org.openbandy.io.SerializationBuffer;
import org.openbandy.io.Serializer;

/**
 * This Event reports the reception of a Message at the MessageScanner.
 * Interested EventSinks may choose to fetch the message by reading
 * either the class name of the message contained serializable or by
 * reading the type of the message.
 * As this type of Event is only used in-node (i.e. it will no be exchanged
 * between nodes), it does not have to be stream serializable.
 * 
 * @version 1.0
 * @author Philipp Bolliger (philipp@bolliger.name)
 */
public class IncomingMessageEvent extends Event {
    
    /* class name of the serializable object that is contained in the incoming message */
    private String contentClassName;
    
    /* type of the message */
    private int messageType = 0;
    
    /* the temporary id of the message in the local message queue */
    private int messageQueueId = 0;

    /**
     * TODO comment!
     *
     * @return
     */
    public String getContentClassName() {
        return contentClassName;
    }

    /**
     * TODO comment!
     *
     * @param contentClassName
     */
    public void setContentClassName(String contentClassName) {
        this.contentClassName = contentClassName;
    }
    
    /**
     * TODO comment!
     *
     * @return
     */
    public int getMessageQueueId() {
        return messageQueueId;
    }

    /**
     * TODO comment!
     *
     * @param messageQueueId
     */
    public void setMessageQueueId(int messageQueueId) {
        this.messageQueueId = messageQueueId;
    }

    /**
     * TODO comment!
     *
     * @return
     */
    public int getMessageType() {
        return messageType;
    }

    /**
     * TODO comment!
     *
     * @param messageType
     */
    public void setMessageType(int messageType) {
        this.messageType = messageType;
    }
    
    /* *************** Methods for Serialization *****************
     * 
     *  Do nothing as this type of Event will not be serialized!
     *  
     *  ********************************************************* */

    public void deserialize(Serializer serializer, SerializationBuffer buffer) {
        /* do nothing */
    }

    public void deserializeSuper(Serializer serializer, SerializationBuffer buffer) {
        /* do nothing */
    }

    public void getAttributeNames(Vector attributes) {
        /* do nothing */
    }

    public Serializable getCopy() {
        /* do nothing */
        return null;
    }

    public String getIdName() {
        /* do nothing */
        return null;
    }

    public int getIdValue() {
        /* do nothing */
        return 0;
    }

    public String getLegibleString() {
        /* do nothing */
        return null;
    }

    public void getReferenceNames(Vector referenceNames) {
        /* do nothing */
    }

    public void getReferences(Vector references) {
        /* do nothing */
    }

    public Serializable getSuperCopy() {
        /* do nothing */
        return null;
    }

    public boolean isEqual(Serializable serializableObject) throws ClassCastException {
        /* do nothing */
        return false;
    }

    public boolean isEqualShallow(Serializable serializableObject) throws ClassCastException {
        /* do nothing */
        return false;
    }

    public void serialize(Serializer serializer, SerializationBuffer buffer) {
        /* do nothing */
    }

    public void setIdValue(int idValue) {
        /* do nothing */
    }

    public void setValidity(int validity) {
        /* do nothing */
    }
    
    

}
