/**
 * Filename: ServerConnection.java (in org.openbandy.net.client)
 * This file is part of the BandyClient project.
 *
 * www.openbandy.org
 */

package org.openbandy.net.client;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InterruptedIOException;

import javax.microedition.io.Connector;
import javax.microedition.io.SocketConnection;

import org.openbandy.service.LogService;
import org.openbandy.service.NotificatorService;

/**
 * TODO describe purpose and usage
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.9
 * 
 */
public class ServerConnection {

	private SocketConnection conn;

	private DataInputStream inputStream = null;

	private DataOutputStream outputStream = null;


	/* ***** Getter Methods ***** */

	public DataInputStream getInputStream() {
		return inputStream;
	}

	public DataOutputStream getOutputStream() {
		return outputStream;
	}

	/* ***** ServerConnection Methods ***** */

	public boolean connect(String host, int port){

		/* check host argument */
		if(host.equals("")){
			LogService.warn(this, "Can not connect, host is not set");
			return false;
		}

		/* check port argument */
		if(!(port != 0)){
			LogService.warn(this, "Can not connect, invalid port");
			return false;
		}

		/* if conn is not null, close it first (got error when trying to reuse open conn) */
		if(conn != null){
			try {
				conn.close();
				LogService.debug(this, "Force closed connection before connect");
			} catch (IOException e) {
				/* do nothing */
			}
		}

		/* connect */
		try{
			conn = (SocketConnection)Connector.open("socket://"+host+":"+port, Connector.READ_WRITE, true);
			if(conn != null){
				outputStream = conn.openDataOutputStream();
				inputStream = conn.openDataInputStream();
				return true;
			}
			else {
				return false;
			}
		}
		catch(SecurityException se){
			/* the user did not allow to connect to server */
			LogService.error(this, "Not authorized to open socket connection", se);

			/* notify user */
			NotificatorService.errorAlert("Not authorized to open socket connection");
		}
		catch(InterruptedIOException iioe){
			/* the connection timed out */
			LogService.error(this, "Connection to "+host+":"+port+" timed out", iioe);

			/* notify user */
			NotificatorService.errorAlert("Connection to "+host+":"+port+" timed out");
		}
		catch(IOException ioe){
			LogService.error(this, "Can not open connection to "+host+":"+port, ioe);

			/* notify user */
			NotificatorService.errorAlert("Can not open connection to "+host+":"+port);
		}
		catch(ClassCastException cce){
			LogService.error(this, "Can not open connection to "+host+":"+port, cce);

			/* notify user */
			NotificatorService.errorAlert("Can not open connection to "+host+":"+port);
		}
		return false;
	}

	public void close(){
		try{
			if(conn != null){
				conn.close();
				LogService.debug(this, "Closed connection");
			}
		}catch(IOException ioe){
			LogService.error(this, "Can not close connection", ioe);
		}
	}
}
